@@ -130,6 +130,7 @@
  (:structure in-addr ("struct in_addr"
                       ((array (unsigned 8)) addr "u_int32_t" "s_addr")))
  (:structure sockaddr-in ("struct sockaddr_in"
+                          #+darwin ((unsigned 8) len "__uint8_t" "sin_len")
                           (integer family "sa_family_t" "sin_family")
                           ;; These two could be in-port-t and
                           ;; in-addr-t, but then we'd throw away the
@@ -287,8 +288,6 @@
  (:integer EAI-SERVICE "EAI_SERVICE")
  #-freebsd
  (:integer EAI-ADDRFAMILY "EAI_ADDRFAMILY")
- #-freebsd
- (:integer EAI-NODATA "EAI_NODATA")
  (:integer EAI-MEMORY "EAI_MEMORY")
  (:integer EAI-FAIL "EAI_FAIL")
  (:integer EAI-AGAIN "EAI_AGAIN")
