@@ -54,9 +54,7 @@
   (inst lea edi (make-ea :dword :base ebx :disp (- n-word-bytes)))
   (inst rep)
   (inst movs :dword)
-
-  ;; solaris requires DF being zero.
-  #!+sunos (inst cld)
+  (inst cld)
 
   ;; Restore the count.
   (inst mov ecx edx)
@@ -153,9 +151,7 @@
   (inst sub esi (fixnumize 1))
   (inst rep)
   (inst movs :dword)
-
-  ;; solaris requires DF being zero.
-  #!+sunos (inst cld)
+  (inst cld)
 
   ;; Load the register arguments carefully.
   (loadw edx ebp-tn -1)
@@ -206,7 +202,7 @@
 
   LOOP
 
-  (let ((error (generate-error-code nil unseen-throw-tag-error target)))
+  (let ((error (generate-error-code nil 'unseen-throw-tag-error target)))
     (inst or catch catch)               ; check for NULL pointer
     (inst jmp :z error))
 
@@ -234,7 +230,7 @@
                           (:temp uwp unsigned-reg esi-offset))
   (declare (ignore start count))
 
-  (let ((error (generate-error-code nil invalid-unwind-error)))
+  (let ((error (generate-error-code nil 'invalid-unwind-error)))
     (inst or block block)               ; check for NULL pointer
     (inst jmp :z error))
 
@@ -279,7 +275,7 @@
                           (:arg count (any-reg descriptor-reg) ecx-offset))
   (declare (ignore start count))
 
-  (let ((error (generate-error-code nil invalid-unwind-error)))
+  (let ((error (generate-error-code nil 'invalid-unwind-error)))
     (inst or block block)               ; check for NULL pointer
     (inst jmp :z error))
 
