@@ -619,9 +619,6 @@
 
 (define-condition simple-error (simple-condition error) ())
 
-;;; not specified by ANSI, but too useful not to have around.
-(define-condition simple-style-warning (simple-condition style-warning) ())
-
 (define-condition storage-condition (serious-condition) ())
 
 (define-condition type-error (error)
@@ -634,6 +631,8 @@
              (type-error-datum condition)
              (type-error-expected-type condition)))))
 
+;;; not specified by ANSI, but too useful not to have around.
+(define-condition simple-style-warning (simple-condition style-warning) ())
 (define-condition simple-type-error (simple-condition type-error) ())
 
 (define-condition program-error (error) ())
@@ -815,7 +814,8 @@
               <http://sbcl.sourceforge.net/>.~:@>"
              '((fmakunbound 'compile))))))
 
-(define-condition simple-storage-condition (storage-condition simple-condition) ())
+(define-condition simple-storage-condition (storage-condition simple-condition)
+  ())
 
 ;;; a condition for use in stubs for operations which aren't supported
 ;;; on some platforms
@@ -900,6 +900,9 @@
           (unless (null (cdr rs))
             (terpri s)))))))
 
+(define-condition simple-reference-error (reference-condition simple-error)
+  ())
+
 (define-condition duplicate-definition (reference-condition warning)
   ((name :initarg :name :reader duplicate-definition-name))
   (:report (lambda (c s)
@@ -968,6 +971,13 @@
     (format-args-mismatch simple-style-warning)
   ())
 
+(define-condition implicit-generic-function-warning (style-warning)
+  ((name :initarg :name :reader implicit-generic-function-name))
+  (:report
+   (lambda (condition stream)
+     (format stream "~@<Implicitly creating new generic function ~S.~:@>"
+             (implicit-generic-function-name condition)))))
+
 (define-condition extension-failure (reference-condition simple-error)
   ())
 
