@@ -39,19 +39,26 @@
 ;;; from defstruct.lisp
 (in-package "SB!KERNEL")
 (defsetf %instance-ref %instance-set)
+#!-hppa
+(progn
 (defsetf %raw-instance-ref/word %raw-instance-set/word)
 (defsetf %raw-instance-ref/single %raw-instance-set/single)
 (defsetf %raw-instance-ref/double %raw-instance-set/double)
 (defsetf %raw-instance-ref/complex-single %raw-instance-set/complex-single)
 (defsetf %raw-instance-ref/complex-double %raw-instance-set/complex-double)
+)
+#!+hppa
+(progn
 (defsetf %raw-ref-single %raw-set-single)
 (defsetf %raw-ref-double %raw-set-double)
 
 (defsetf %raw-ref-complex-single %raw-set-complex-single)
 (defsetf %raw-ref-complex-double %raw-set-complex-double)
+)
 
 (defsetf %instance-layout %set-instance-layout)
 (defsetf %funcallable-instance-info %set-funcallable-instance-info)
+(defsetf %funcallable-instance-layout %set-funcallable-instance-layout)
 
 ;;; from early-setf.lisp
 (in-package "SB!IMPL")
@@ -129,8 +136,8 @@
 (defsetf sap-ref-double %set-sap-ref-double)
 #!+long-float (defsetf sap-ref-long %set-sap-ref-long)
 #-sb-xc-host (defsetf subseq (sequence start &optional (end nil)) (v)
-	    `(progn (replace ,sequence ,v :start1 ,start :end1 ,end)
-		    ,v))
+            `(progn (replace ,sequence ,v :start1 ,start :end1 ,end)
+                    ,v))
 
 ;;; from fdefinition.lisp
 (in-package "SB!IMPL")
