@@ -154,7 +154,8 @@
 (sb!xc:deftype char-code () `(integer 0 (,sb!xc:char-code-limit)))
 
 ;;; a consed sequence result. If a vector, is a simple array.
-(sb!xc:deftype consed-sequence () '(or list (simple-array * (*))))
+(sb!xc:deftype consed-sequence ()
+  '(or (simple-array * (*)) list extended-sequence))
 
 ;;; the :END arg to a sequence
 (sb!xc:deftype sequence-end () '(or null index))
