@@ -35,8 +35,8 @@
     (values
      (or (gethash extern table)
          (gethash (concatenate 'base-string
-                               #!+(and darwin (or x86 x86-64)) "_ldso_stub__"
-                               #!-(and darwin (or x86 x86-64)) "ldso_stub__"
+                               #!+(and darwin (or x86 x86-64 ppc)) "_ldso_stub__"
+                               #!-(and darwin (or x86 x86-64 ppc)) "ldso_stub__"
                                extern) table)))))
 
 (defun find-foreign-symbol-address (name)
@@ -129,13 +129,11 @@ if the symbol isn't found."
     (when (<= sb!vm:linkage-table-space-start
               addr
               sb!vm:linkage-table-space-end)
-      (maphash (lambda (name-and-datap info)
-                 (let ((table-addr (linkage-info-address info)))
-                   (when (<= table-addr
-                             addr
-                             (+ table-addr sb!vm:linkage-table-entry-size))
-                     (return-from sap-foreign-symbol (car name-and-datap)))))
-               *linkage-info*))
+      (dohash ((name-and-datap info) *linkage-info* :locked t)
+        (let ((table-addr (linkage-info-address info)))
+          (when (and (<= table-addr addr)
+                     (< addr (+ table-addr sb!vm:linkage-table-entry-size)))
+            (return-from sap-foreign-symbol (car name-and-datap))))))
     #!+os-provides-dladdr
     (with-alien ((info (struct dl-info
                                (filename c-string)
