@@ -79,8 +79,8 @@
 ;;; long is another question. This stuff still needs testing.
 (define-alien-routine ("os_context_fpregister_addr" context-float-register-addr)
     (* os-context-register-t)
-  (context (* os-context-t))
-  (index int))
+  (context (* os-context-t) :in)
+  (index int :in))
 
 (defun context-float-register (context index format)
   (declare (type (alien (* os-context-t)) context))
@@ -94,11 +94,19 @@
     (declare (type (alien (* os-context-register-t)) addr))
     (setf (deref addr) (coerce new format))))
 
+(define-alien-routine
+    ("arch_get_fp_control" floating-point-modes) unsigned-int)
+
+(define-alien-routine
+    ("arch_set_fp_control" %floating-point-modes-setter) void (fp unsigned-int :in))
+
+(defun (setf floating-point-modes) (val) (%floating-point-modes-setter val))
+
 ;;; Given a signal context, return the floating point modes word in
 ;;; the same format as returned by FLOATING-POINT-MODES.
 (define-alien-routine ("os_context_fp_control" context-floating-point-modes)
     unsigned-int
-  (context (* os-context-t)))
+  (context (* os-context-t) :in))
 
 ;;;; Internal-error-arguments.
 
