@@ -46,7 +46,8 @@
   (make-hash-table
    ;; EQL testing isn't good enough for generalized function names
    ;; like (SETF FOO).
-   :test 'equal))
+   :test 'equal
+   :synchronized t))
 (defstruct (profile-info (:copier nil))
   (name              (missing-arg) :read-only t)
   (encapsulated-fun  (missing-arg) :type function :read-only t)
@@ -144,7 +145,7 @@
     (values
      ;; ENCAPSULATION-FUN
      (lambda (&more arg-context arg-count)
-       (declare (optimize speed safety))
+       (declare (optimize speed safety sb-c::stack-allocate-dynamic-extent))
        ;; Make sure that we're not recursing infinitely.
        (when (boundp '*computing-profiling-data-for*)
          (unprofile-all) ; to avoid further recursion
@@ -157,8 +158,8 @@
        (let ((dticks 0)
              (dconsing 0)
              (inner-enclosed-profiles 0))
-         (declare (type unsigned-byte dticks dconsing))
-         (declare (type unsigned-byte inner-enclosed-profiles))
+         (declare (type unsigned-byte dticks dconsing inner-enclosed-profiles)
+                  (dynamic-extent dticks dconsing inner-enclosed-profiles))
          (aver (typep dticks 'unsigned-byte))
          (aver (typep dconsing 'unsigned-byte))
          (aver (typep inner-enclosed-profiles 'unsigned-byte))
@@ -309,13 +310,14 @@
       `(unprofile-all)))
 
 (defun unprofile-all ()
-  (dohash (name profile-info *profiled-fun-name->info*)
+  (dohash ((name profile-info) *profiled-fun-name->info*
+           :locked t)
     (declare (ignore profile-info))
     (unprofile-1-fun name)))
 
 (defun reset ()
   "Reset the counters for all profiled functions."
-  (dohash (name profile-info *profiled-fun-name->info*)
+  (dohash ((name profile-info) *profiled-fun-name->info* :locked t)
     (declare (ignore name))
     (funcall (profile-info-clear-stats-fun profile-info))))
 
@@ -358,7 +360,7 @@ Lisp process."
           (compute-overhead)))
   (let ((time-info-list ())
         (no-call-name-list ()))
-    (dohash (name pinfo *profiled-fun-name->info*)
+    (dohash ((name pinfo) *profiled-fun-name->info* :locked t)
       (unless (eq (fdefinition name)
                   (profile-info-encapsulation-fun pinfo))
         (warn "Function ~S has been redefined, so times may be inaccurate.~@
