@@ -31,8 +31,9 @@
                         (if key (%coerce-callable-to-fun key) #'identity))
       (let ((key-fun-or-nil (and key (%coerce-callable-to-fun key))))
         (with-array-data ((vector (the vector sequence))
-                          (start 0)
-                          (end (length sequence)))
+                          (start)
+                          (end)
+                          :check-fill-pointer t)
           (sort-vector vector start end predicate-fun key-fun-or-nil))
         sequence)
       (apply #'sb!sequence:sort sequence predicate args))))
@@ -239,8 +240,8 @@
                 (simple-vector ,temp))
        (when (> ,vector-len ,temp-len)
          (setf ,temp (make-array (max ,vector-len
-                                      (min most-positive-fixnum
-                                           (+ ,temp-len ,temp-len))))
+                                      (min (truncate array-dimension-limit 2)
+                                           (logand most-positive-fixnum (+ ,temp-len ,temp-len)))))
                *merge-sort-temp-vector* ,temp))
        ;; Rebind, in case PRED or KEY calls STABLE-SORT. This is also
        ;; interrupt safe: we bind before we put any data of our own in
@@ -296,10 +297,6 @@
 
 ) ; EVAL-when
 
-;;; temporary vector for stable sorting vectors, allocated for each new thread
-(defvar *merge-sort-temp-vector* (vector))
-(declaim (simple-vector *merge-sort-temp-vector*))
-
 (defun stable-sort-simple-vector (vector pred key)
   (declare (type simple-vector vector)
            (type function pred)
