@@ -61,6 +61,10 @@
   ;; in the safe code. If a function MUST signal errors, then it is
   ;; not unsafely-flushable even if it is movable or foldable.
   unsafely-flushable
+  ;; return value is important, and ignoring it is probably a mistake.
+  ;; Unlike the other attributes, this is used only for style
+  ;; warnings and has no effect on optimization.
+  important-result
   ;; may be moved with impunity. Has no side effects except possibly
   ;; consing, and is affected only by its arguments.
   ;;
@@ -81,7 +85,12 @@
   ;; The function does explicit argument type checking, so the
   ;; declared type should not be asserted when a definition is
   ;; compiled.
-  explicit-check)
+  explicit-check
+  ;; The function should always be translated by a VOP (i.e. it should
+  ;; should never be converted into a full call).  This is used strictly
+  ;; as a consistency checking mechanism inside the compiler during IR2
+  ;; transformation.
+  always-translatable)
 
 (defstruct (fun-info #-sb-xc-host (:pure t))
   ;; boolean attributes of this function.
@@ -100,6 +109,9 @@
   ;; further optimiz'ns) is backwards from the return convention for
   ;; transforms. -- WHN 19990917
   (optimizer nil :type (or function null))
+  ;; a function computing the constant or literal arguments which are
+  ;; destructively modified by the call.
+  (destroyed-constant-args nil :type (or function null))
   ;; If true, a special-case LTN annotation method that is used in
   ;; place of the standard type/policy template selection. It may use
   ;; arbitrary code to choose a template, decide to do a full call, or
@@ -157,18 +169,18 @@
 ;;; Grab the FUN-INFO and enter the function, replacing any old
 ;;; one with the same type and note.
 (declaim (ftype (function (t list function &optional (or string null)
-			     (member t nil))
-			  *)
-		%deftransform))
+                             (member t nil))
+                          *)
+                %deftransform))
 (defun %deftransform (name type fun &optional note important)
   (let* ((ctype (specifier-type type))
-	 (note (or note "optimize"))
-	 (info (fun-info-or-lose name))
-	 (old (find-if (lambda (x)
-			 (and (type= (transform-type x) ctype)
-			      (string-equal (transform-note x) note)
-			      (eq (transform-important x) important)))
-		       (fun-info-transforms info))))
+         (note (or note "optimize"))
+         (info (fun-info-or-lose name))
+         (old (find-if (lambda (x)
+                         (and (type= (transform-type x) ctype)
+                              (string-equal (transform-note x) note)
+                              (eq (transform-important x) important)))
+                       (fun-info-transforms info))))
     (cond (old
            (style-warn "Overwriting ~S" old)
            (setf (transform-function old) fun
@@ -182,31 +194,33 @@
 ;;; Make a FUN-INFO structure with the specified type, attributes
 ;;; and optimizers.
 (declaim (ftype (function (list list attributes &key
-				(:derive-type (or function null))
-				(:optimizer (or function null)))
-			  *)
-		%defknown))
-(defun %defknown (names type attributes &key derive-type optimizer)
+                                (:derive-type (or function null))
+                                (:optimizer (or function null))
+                                (:destroyed-constant-args (or function null)))
+                          *)
+                %defknown))
+(defun %defknown (names type attributes &key derive-type optimizer destroyed-constant-args)
   (let ((ctype (specifier-type type))
-	(info (make-fun-info :attributes attributes
+        (info (make-fun-info :attributes attributes
                              :derive-type derive-type
-                             :optimizer optimizer))
-	(target-env *info-environment*))
+                             :optimizer optimizer
+                             :destroyed-constant-args destroyed-constant-args))
+        (target-env *info-environment*))
     (dolist (name names)
       (let ((old-fun-info (info :function :info name)))
-	(when old-fun-info
-	  ;; This is handled as an error because it's generally a bad
-	  ;; thing to blow away all the old optimization stuff. It's
-	  ;; also a potential source of sneaky bugs:
-	  ;;    DEFKNOWN FOO
-	  ;;    DEFTRANSFORM FOO
-	  ;;    DEFKNOWN FOO ; possibly hidden inside some macroexpansion
-	  ;;    ; Now the DEFTRANSFORM doesn't exist in the target Lisp.
-	  ;; However, it's continuable because it might be useful to do
-	  ;; it when testing new optimization stuff interactively.
-	  (cerror "Go ahead, overwrite it."
-		  "~@<overwriting old FUN-INFO ~2I~_~S ~I~_for ~S~:>"
-		  old-fun-info name)))
+        (when old-fun-info
+          ;; This is handled as an error because it's generally a bad
+          ;; thing to blow away all the old optimization stuff. It's
+          ;; also a potential source of sneaky bugs:
+          ;;    DEFKNOWN FOO
+          ;;    DEFTRANSFORM FOO
+          ;;    DEFKNOWN FOO ; possibly hidden inside some macroexpansion
+          ;;    ; Now the DEFTRANSFORM doesn't exist in the target Lisp.
+          ;; However, it's continuable because it might be useful to do
+          ;; it when testing new optimization stuff interactively.
+          (cerror "Go ahead, overwrite it."
+                  "~@<overwriting old FUN-INFO ~2I~_~S ~I~_for ~S~:>"
+                  old-fun-info name)))
       (setf (info :function :type name target-env) ctype)
       (setf (info :function :where-from name target-env) :declared)
       (setf (info :function :kind name target-env) :function)
@@ -221,11 +235,11 @@
 (declaim (ftype (sfunction (t) fun-info) fun-info-or-lose))
 (defun fun-info-or-lose (name)
   (let (;; FIXME: Do we need this rebinding here? It's a literal
-	;; translation of the old CMU CL rebinding to
-	;; (OR *BACKEND-INFO-ENVIRONMENT* *INFO-ENVIRONMENT*),
-	;; and it's not obvious whether the rebinding to itself is
-	;; needed that SBCL doesn't need *BACKEND-INFO-ENVIRONMENT*.
-	(*info-environment* *info-environment*))
+        ;; translation of the old CMU CL rebinding to
+        ;; (OR *BACKEND-INFO-ENVIRONMENT* *INFO-ENVIRONMENT*),
+        ;; and it's not obvious whether the rebinding to itself is
+        ;; needed that SBCL doesn't need *BACKEND-INFO-ENVIRONMENT*.
+        (*info-environment* *info-environment*))
     (let ((old (info :function :info name)))
       (unless old (error "~S is not a known function." name))
       (setf (info :function :info name) (copy-fun-info old)))))
@@ -249,8 +263,8 @@
 (defun result-type-float-contagion (call)
   (declare (type combination call))
   (reduce #'numeric-contagion (combination-args call)
-	  :key #'lvar-type
-	  :initial-value (specifier-type 'single-float)))
+          :key #'lvar-type
+          :initial-value (specifier-type 'single-float)))
 
 ;;; Return a closure usable as a derive-type method for accessing the
 ;;; N'th argument. If arg is a list, result is a list. If arg is a
@@ -260,13 +274,13 @@
     (declare (type combination call))
     (let ((lvar (nth (1- n) (combination-args call))))
       (when lvar
-	(let ((type (lvar-type lvar)))
-	  (if (array-type-p type)
-	      (specifier-type
-	       `(vector ,(type-specifier (array-type-element-type type))))
-	      (let ((ltype (specifier-type 'list)))
-		(when (csubtypep type ltype)
-		  ltype))))))))
+        (let ((type (lvar-type lvar)))
+          (if (array-type-p type)
+              (specifier-type
+               `(vector ,(type-specifier (array-type-element-type type))))
+              (let ((ltype (specifier-type 'list)))
+                (when (csubtypep type ltype)
+                  ltype))))))))
 
 ;;; Derive the type to be the type specifier which is the Nth arg.
 (defun result-type-specifier-nth-arg (n)
@@ -274,7 +288,7 @@
     (declare (type combination call))
     (let ((lvar (nth (1- n) (combination-args call))))
       (when (and lvar (constant-lvar-p lvar))
-	(careful-specifier-type (lvar-value lvar))))))
+        (careful-specifier-type (lvar-value lvar))))))
 
 ;;; Derive the type to be the type specifier which is the Nth arg,
 ;;; with the additional restriptions noted in the CLHS for STRING and
@@ -285,35 +299,75 @@
     (declare (type combination call))
     (let ((lvar (nth (1- n) (combination-args call))))
       (when (and lvar (constant-lvar-p lvar))
-	(let* ((specifier (lvar-value lvar))
-	       (lspecifier (if (atom specifier) (list specifier) specifier)))
-	  (cond
-	    ((eq (car lspecifier) 'string)
-	     (destructuring-bind (string &rest size)
-		 lspecifier
-	       (declare (ignore string))
-	       (careful-specifier-type
-		`(vector character ,@(when size size)))))
-	    ((eq (car lspecifier) 'simple-string)
-	     (destructuring-bind (simple-string &rest size)
-		 lspecifier
-	       (declare (ignore simple-string))
-	       (careful-specifier-type
-		`(simple-array character ,@(if size (list size) '((*)))))))
-	    (t
-	     (let ((ctype (careful-specifier-type specifier)))
-	       (if (and (array-type-p ctype)
-			(eq (array-type-specialized-element-type ctype)
-			    *wild-type*))
-		   ;; I don't think I'm allowed to modify what I get
-		   ;; back from SPECIFIER-TYPE; it is, after all,
-		   ;; cached.  Better copy it, then.
-		   (let ((real-ctype (copy-structure ctype)))
-		     (setf (array-type-element-type real-ctype)
-			   *universal-type*
-			   (array-type-specialized-element-type real-ctype)
-			   *universal-type*)
-		     real-ctype)
-		   ctype)))))))))
+        (let* ((specifier (lvar-value lvar))
+               (lspecifier (if (atom specifier) (list specifier) specifier)))
+          (cond
+            ((eq (car lspecifier) 'string)
+             (destructuring-bind (string &rest size)
+                 lspecifier
+               (declare (ignore string))
+               (careful-specifier-type
+                `(vector character ,@(when size size)))))
+            ((eq (car lspecifier) 'simple-string)
+             (destructuring-bind (simple-string &rest size)
+                 lspecifier
+               (declare (ignore simple-string))
+               (careful-specifier-type
+                `(simple-array character ,@(if size (list size) '((*)))))))
+            (t
+             (let ((ctype (careful-specifier-type specifier)))
+               (if (and (array-type-p ctype)
+                        (eq (array-type-specialized-element-type ctype)
+                            *wild-type*))
+                   ;; I don't think I'm allowed to modify what I get
+                   ;; back from SPECIFIER-TYPE; it is, after all,
+                   ;; cached.  Better copy it, then.
+                   (let ((real-ctype (copy-structure ctype)))
+                     (setf (array-type-element-type real-ctype)
+                           *universal-type*
+                           (array-type-specialized-element-type real-ctype)
+                           *universal-type*)
+                     real-ctype)
+                   ctype)))))))))
+
+(defun remove-non-constants-and-nils (fun)
+  (lambda (list)
+    (remove-if-not #'lvar-value
+                   (remove-if-not #'constant-lvar-p (funcall fun list)))))
+
+;;; FIXME: bad name (first because it uses 1-based indexing; second
+;;; because it doesn't get the nth constant arguments)
+(defun nth-constant-args (&rest indices)
+  (lambda (list)
+    (let (result)
+      (do ((i 1 (1+ i))
+           (list list (cdr list))
+           (indices indices))
+          ((null indices) (nreverse result))
+        (when (= i (car indices))
+          (when (constant-lvar-p (car list))
+            (push (car list) result))
+          (setf indices (cdr indices)))))))
+
+;;; FIXME: a number of the sequence functions not only do not destroy
+;;; their argument if it is empty, but also leave it alone if :start
+;;; and :end bound a null sequence, or if :count is 0.  This test is a
+;;; bit complicated to implement, verging on the impossible, but for
+;;; extra points (fill #\1 "abc" :start 0 :end 0) should not cause a
+;;; warning.
+(defun nth-constant-nonempty-sequence-args (&rest indices)
+  (lambda (list)
+    (let (result)
+      (do ((i 1 (1+ i))
+           (list list (cdr list))
+           (indices indices))
+          ((null indices) (nreverse result))
+        (when (= i (car indices))
+          (when (constant-lvar-p (car list))
+            (let ((value (lvar-value (car list))))
+              (unless (or (typep value 'null)
+                          (typep value '(vector * 0)))
+                (push (car list) result))))
+          (setf indices (cdr indices)))))))
 
 (/show0 "knownfun.lisp end of file")
