@@ -305,7 +305,7 @@
                                        ,@cases)))))
           (aver (null type))
           (inst call (make-fixup dst :assembly-routine)))
-        (pseudo-atomic
+        (maybe-pseudo-atomic stack-allocate-p
          (allocation result (pad-data-block words) node stack-allocate-p)
          (inst lea result (make-ea :byte :base result :disp lowtag))
          (when type
