@@ -17,7 +17,7 @@
 
 ;;;; compiler constants
 
-(defconstant +backend-fasl-file-implementation+ :x86)
+(def!constant +backend-fasl-file-implementation+ :x86)
 
 (setf *backend-register-save-penalty* 3)
 
@@ -27,7 +27,13 @@
 ;;; code for it, but mostly we need it for GENESIS, which doesn't in
 ;;; general have our C runtime code running to ask, so instead we set
 ;;; it by hand. -- WHN 2001-04-15
-(setf *backend-page-size* 4096)
+;;;
+;;; Actually any information that we can retrieve C-side would be
+;;; useless in SBCL, since it's possible for otherwise binary
+;;; compatible systems to return different values for getpagesize().
+;;; -- JES, 2007-01-06
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (setf *backend-page-size* 4096))
 ;;; comment from CMU CL:
 ;;;
 ;;;   in case we ever wanted to do this for Windows NT..
@@ -38,3 +44,7 @@
 ;;;   Effectively, the page size is 64K.
 ;;;
 ;;;   would be: (setf *backend-page-size* 65536)
+
+;;; The size in bytes of the GENCGC pages. Should be a multiple of the
+;;; architecture code size.
+(def!constant gencgc-page-size *backend-page-size*)
