@@ -13,25 +13,8 @@
 
 ;;;; test generation utilities
 
-;;; Emit the most compact form of the test immediate instruction,
-;;; using an 8 bit test when the immediate is only 8 bits and the
-;;; value is one of the four low registers (eax, ebx, ecx, edx) or the
-;;; control stack.
 (defun generate-fixnum-test (value)
-  (let ((offset (tn-offset value)))
-    (cond ((and (sc-is value any-reg descriptor-reg)
-                (or (= offset eax-offset) (= offset ebx-offset)
-                    (= offset ecx-offset) (= offset edx-offset)))
-           (inst test (make-random-tn :kind :normal
-                                      :sc (sc-or-lose 'byte-reg)
-                                      :offset offset)
-                 3))
-          ((sc-is value control-stack)
-           (inst test (make-ea :byte :base ebp-tn
-                               :disp (- (* (1+ offset) n-word-bytes)))
-                 3))
-          (t
-           (inst test value 3)))))
+  (emit-optimized-test-inst value 3))
 
 (defun %test-fixnum (value target not-p)
   (generate-fixnum-test value)
@@ -214,7 +197,7 @@
            `((define-vop (,check-name ,(intern (concatenate 'string prefix "CHECK-TYPE")))
                (:generator ,cost
                  (let ((err-lab
-                        (generate-error-code vop ,error-code value)))
+                        (generate-error-code vop ',error-code value)))
                    (test-type value err-lab t (,@type-codes))
                    (move result value))))))
        ,@(when ptype
@@ -254,7 +237,7 @@
 (define-vop (check-signed-byte-32 check-type)
   (:generator 45
     (let ((nope (generate-error-code vop
-                                     object-not-signed-byte-32-error
+                                     'object-not-signed-byte-32-error
                                      value)))
       (generate-fixnum-test value)
       (inst jmp :e yep)
@@ -319,7 +302,7 @@
 (define-vop (check-unsigned-byte-32 check-type)
   (:generator 45
     (let ((nope
-           (generate-error-code vop object-not-unsigned-byte-32-error value))
+           (generate-error-code vop 'object-not-unsigned-byte-32-error value))
           (yep (gen-label))
           (fixnum (gen-label))
           (single-word (gen-label)))
@@ -376,7 +359,7 @@
 
 (define-vop (check-symbol check-type)
   (:generator 12
-    (let ((error (generate-error-code vop object-not-symbol-error value)))
+    (let ((error (generate-error-code vop 'object-not-symbol-error value)))
       (inst cmp value nil-value)
       (inst jmp :e drop-thru)
       (test-type value error t (symbol-header-widetag)))
@@ -394,7 +377,7 @@
 
 (define-vop (check-cons check-type)
   (:generator 8
-    (let ((error (generate-error-code vop object-not-cons-error value)))
+    (let ((error (generate-error-code vop 'object-not-cons-error value)))
       (inst cmp value nil-value)
       (inst jmp :e error)
       (test-type value error t (list-pointer-lowtag))
