@@ -27,7 +27,7 @@
 (!fix-ensure-accessor-specializers)
 (compute-standard-slot-locations)
 (dolist (s '(condition structure-object))
-  (dohash (k v (classoid-subclasses (find-classoid s)))
+  (dohash ((k v) (classoid-subclasses (find-classoid s)))
     (find-class (classoid-name k))))
 (setq *boot-state* 'complete)
 
