@@ -51,16 +51,6 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-/* KLUDGE: Avoid double definition of boolean by rpcndr.h included via
- * shlobj.h.
- *
- * FIXME: We should probably arrange to use the rpcndr.h boolean on Windows,
- * or get rid of our own boolean type.
- */
-#define boolean rpcndr_boolean
-#include <shlobj.h>
-#undef boolean
-
 #include <math.h>
 #include <float.h>
 
@@ -428,7 +418,7 @@ handle_exception(EXCEPTION_RECORD *exception_record,
         _clearfp();
 
         /* Call into lisp to handle things. */
-        funcall2(SymbolFunction(HANDLE_WIN32_EXCEPTION), context_sap,
+        funcall2(StaticSymbolFunction(HANDLE_WIN32_EXCEPTION), context_sap,
                  exception_record_sap);
 
         /* If Lisp doesn't nlx, we need to put things back. */
