@@ -32,6 +32,10 @@
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
+#include <errno.h>
+#include <limits.h>
+#include <fcntl.h>
+
 #ifndef LISP_FEATURE_WIN32
 #include <pwd.h>
 #include <sys/wait.h>
@@ -39,8 +43,14 @@
 #endif
 #include <stdio.h>
 
+#if defined(LISP_FEATURE_WIN32)
+#define WIN32_LEAN_AND_MEAN
+#include <errno.h>
+#endif
+
 #include "runtime.h"
 #include "util.h"
+#include "wrap.h"
 
 /* Although it might seem as though this should be in some standard
    Unix header, according to Perry E. Metzger, in a message on
@@ -150,76 +160,45 @@ wrapped_readlink(char *path)
 }
 #endif
 
+/*
+ * realpath(3), including a wrapper for Windows.
+ */
+char * sb_realpath (char *path)
+{
+#ifndef LISP_FEATURE_WIN32
+    char *ret;
+    int errnum;
+
+    if ((ret = calloc(PATH_MAX, sizeof(char))) == NULL)
+        return NULL;
+    if (realpath(path, ret) == NULL) {
+        errnum = errno;
+        free(ret);
+        errno = errnum;
+        return NULL;
+    }
+    return(ret);
+#else
+    char *ret;
+    char *cp;
+    int errnum;
+
+    if ((ret = calloc(MAX_PATH, sizeof(char))) == NULL)
+        return NULL;
+    if (GetFullPathName(path, MAX_PATH, ret, cp) == 0) {
+        errnum = errno;
+        free(ret);
+        errno = errnum;
+        return NULL;
+    }
+    return(ret);
+#endif
+}
+
 /*
  * stat(2) stuff
  */
 
-/* As of 0.6.12, the FFI can't handle 64-bit values. For now, we use
- * these munged-to-32-bits values for might-be-64-bit slots of
- * stat_wrapper as a workaround, so that at least we can still work
- * when values are small.
- *
- * FIXME: But of course we should fix the FFI so that we can use the
- * actual 64-bit values instead.  In fact, we probably have by now
- * (2003-10-03) on all working platforms except MIPS and HPPA; if some
- * motivated spark would simply fix those, this hack could go away.
- * -- CSR, 2003-10-03
- *
- * Some motivated spark fixed MIPS. -- ths, 2005-10-06 */
-
-#if defined(LISP_FEATURE_LARGEFILE) && !defined(LISP_FEATURE_MIPS)
-typedef dev_t ffi_dev_t;
-typedef off_t ffi_off_t;
-#elif defined(LISP_FEATURE_MIPS)
-typedef unsigned long ffi_dev_t; /* Linux/MIPS struct stat doesn't use dev_t */
-typedef off_t ffi_off_t;
-#elif defined(LISP_FEATURE_DARWIN)
-typedef dev_t ffi_dev_t;
-typedef off_t ffi_off_t;
-#else
-typedef u32 ffi_dev_t; /* since Linux dev_t can be 64 bits */
-typedef u32 ffi_off_t; /* since OpenBSD 2.8 st_size is 64 bits */
-#endif
-
-#ifdef LISP_FEATURE_OS_PROVIDES_BLKSIZE_T
-typedef blksize_t ffi_blksize_t;
-#else
-typedef unsigned long ffi_blksize_t;
-#endif
-
-/* a representation of stat(2) results which doesn't depend on CPU or OS */
-struct stat_wrapper {
-    /* KLUDGE: The verbose wrapped_st_ prefixes are to protect us from
-     * the C preprocessor as wielded by the fiends of OpenBSD, who do
-     * things like
-     *    #define st_atime        st_atimespec.tv_sec
-     * I remember when I was young and innocent, I read about how the
-     * C preprocessor isn't to be used to globally munge random
-     * lowercase symbols like this, because things like this could
-     * happen, and I nodded sagely. But now I know better.:-| This is
-     * another entry for Dan Barlow's ongoing episodic rant about C
-     * header files, I guess.. -- WHN 2001-05-10 */
-    ffi_dev_t     wrapped_st_dev;         /* device */
-    ino_t         wrapped_st_ino;         /* inode */
-    mode_t        wrapped_st_mode;        /* protection */
-#ifndef LISP_FEATURE_WIN32
-    nlink_t       wrapped_st_nlink;       /* number of hard links */
-    uid_t         wrapped_st_uid;         /* user ID of owner */
-    gid_t         wrapped_st_gid;         /* group ID of owner */
-#else
-    short         wrapped_st_nlink;       /* Win32 doesn't have nlink_t */
-    short         wrapped_st_uid;         /* Win32 doesn't have st_uid */
-    short         wrapped_st_gid;         /* Win32 doesn't have st_gid */
-#endif
-    ffi_dev_t     wrapped_st_rdev;        /* device type (if inode device) */
-    ffi_off_t     wrapped_st_size;        /* total size, in bytes */
-    ffi_blksize_t wrapped_st_blksize;     /* blocksize for filesystem I/O */
-    unsigned long wrapped_st_blocks;      /* number of blocks allocated */
-    time_t        wrapped_st_atime;       /* time_t of last access */
-    time_t        wrapped_st_mtime;       /* time_t of last modification */
-    time_t        wrapped_st_ctime;       /* time_t of last change */
-};
-
 static void
 copy_to_stat_wrapper(struct stat_wrapper *to, struct stat *from)
 {
@@ -303,6 +282,45 @@ fstat_wrapper(int filedes, struct stat_wrapper *buf)
         copy_to_stat_wrapper(buf, &real_buf);
     return ret;
 }
+
+/* A wrapper for mkstemp(3), for two reasons: (1) mkstemp does not
+   exist on Windows; (2) by passing down a mode_t, we don't need a
+   binding to chmod in SB-UNIX, and need not concern ourselves with
+   umask issues if we want to use mkstemp to make new files in
+   OPEN. */
+int sb_mkstemp (char *template, mode_t mode) {
+#ifdef LISP_FEATURE_WIN32
+#define PATHNAME_BUFFER_SIZE MAX_PATH
+#define MKTEMP _mktemp
+#else
+#define PATHNAME_BUFFER_SIZE PATH_MAX
+#define MKTEMP mktemp
+#endif
+  int fd;
+  char buf[PATHNAME_BUFFER_SIZE];
+
+  while (1) {
+    /* Fruit fallen from the tree: for people who like
+       microoptimizations, we might not need to copy the whole
+       template on every loop, but only the last several characters.
+       But I didn't feel like testing the boundary cases in Windows's
+       _mktemp. */
+    strncpy(buf, template, PATHNAME_BUFFER_SIZE);
+    buf[PATHNAME_BUFFER_SIZE-1]=0; /* force NULL-termination */
+    if (MKTEMP(buf)) {
+      if ((fd=open(buf, O_CREAT|O_EXCL|O_RDWR, mode))!=-1) {
+        strcpy(template, buf);
+        return (fd);
+      } else
+        if (errno != EEXIST)
+          return (-1);
+    } else
+      return (-1);
+  }
+#undef MKTEMP
+#undef PATHNAME_BUFFER_SIZE
+}
+
 
 /*
  * getpwuid() stuff
@@ -341,7 +359,7 @@ uid_homedir(uid_t uid)
         } else {
             char *result = malloc(len + 2);
             if (result) {
-                int nchars = sprintf(result,"%s/",p->pw_dir);
+                unsigned int nchars = sprintf(result,"%s/",p->pw_dir);
                 if (nchars == len + 1) {
                     return result;
                 } else {
@@ -372,7 +390,6 @@ wrapped_environ()
 }
 
 #ifdef LISP_FEATURE_WIN32
-#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <time.h>
 /*
